#!/usr/bin/perl -w

# =================================================================
#
# Name    : Calc_Stt
#
# Purpose : Column based statistics calculating tool
#
# Author  : J. Dousa
#
# Created : 10 Feb 2008
# 
# Changes : NEED TO RESOLVE A PROBLEM WITH ARGV and space in outfmt="asd  asdf asf " !!
#
# =================================================================

package Calc_Stt;

use Exporter;
use Getopt::Long;
use strict;

@Calc_Stt::ISA    = qw(Exporter);
@Calc_Stt::EXPORT = qw(calc_stat);

# -------------
sub calc_stat {

 # get arguments if called as a perl-function
 if( defined $_[0] ){
   $_[0] =~ s/^\s*//g;
   ( @ARGV ) = split '\s+', $_[0];
 }
 my( @ALL, @INP );
 my( $Sum,    $SumA,  $Sum2,    $SumV,   $SumW,   $RedSum, $RedSum2 ) = ( 0, 0, 0, 0, 0, 0, 0);
 my( $Median, $Mean,  $AbsMean, $AbsMedi,$StdDev, $Rms,    $WRms    ) = ( 0, 0, 0, 0, 0, 0, 0);
 my( $OkVal,  $NoVal, $OkValW,  $ExVal,  $TotVal, $MidIdx           ) = ( 0, 0, 0, 0, 0, 0   );
 my( $OkPc,   $NoPc,  $ExPc,    $Iter,   $ConvEpo                   ) = ( 0, 0, 0, 0, undef  );

 my $iFile   = undef;
 my $oFile   = undef;
 my $CnfInt  =   3.0; # confident interval
 my $CnvLim  =   0.1; # convergence limit
 my $ColVal  =     0;
 my $AbsVal  =     0; # if values are already absolute 
 my $ColRms  =     0;
 my $ColRef  =     0;
 my $ValRef  = undef;
 my $ColCpy  =    "";
 my $RunStt  =    -1;
 my $MaxIter =    10;
 my $verb    =     0;
 my $help    =     0;
 my $outFmt  =    "";
    $outFmt .= "%n# Median  = %+12.5fMedian    %tdat[%MidIdx]";
    $outFmt .= "%n# AbsMedi = %+12.5fAbsMedian               ";
    $outFmt .= "%n# Mean    = %+12.5fMean      %tsum/nVal";
    $outFmt .= "%n# AbsMean = %+12.5fAbsMean   %tsumA/nVal";
    $outFmt .= "%n# Rms     = %+12.5fRms       %tsqrt(sum^2/(nVal))";
    $outFmt .= "%n# WRms    = %+12.5fWRms      %tsqrt([pvv]/[p])";
    $outFmt .= "%n# StdDev  = %+12.5fStdDev    %tsqrt(redsum^2/nVal-1)";
    $outFmt .= "%n";    
    $outFmt .= "%n# Iter    = %6iIter  %t%t#       iterations";
    $outFmt .= "%n# Total   = %6iTotVal             %t#       all values";
    $outFmt .= "%n# OkVal   = %6iOkVal  %5.1fOkPc % %t#     valid values";
    $outFmt .= "%n# NoVal   = %6iNoVal  %5.1fNoPc % %t# not valid values";
    $outFmt .= "%n# nExcl   = %6iExVal  %5.1fExPc % %t#  excluded values";
    $outFmt .= "%n%n";

 GetOptions(   'ifile=s'  => \$iFile,
               'ofile=s'  => \$oFile,
              'outfmt=s'  => \$outFmt,
              'cnvlim=s'  => \$CnvLim,
              'cnfint=s'  => \$CnfInt,
              'column=s'  => \$ColVal,
              'rmscol=s'  => \$ColRms,
              'refcol=s'  => \$ColRef,
              'refval=s'  => \$ValRef,
              'cpycol=s'  => \$ColCpy,
                 'run=i'  => \$RunStt,
              'absval'    => \$AbsVal,
               'verb+'    => \$verb,
                'help'    => \$help  ) or &Usage(""); # pod2usage(1);

# if( $help or ! defined "$iFile" or ! -f $iFile){ &Usage(""); }
 if( $help ){ &Usage(""); }

 my @ColVal = ( $ColVal =~ /:/ ) ? split ':', $ColVal : ($ColVal, -1) ; # col, MaxVal 
 my @ColRms = ( $ColRms =~ /:/ ) ? split ':', $ColRms : ($ColRms, -1) ; # col, MaxRms
 my @ColRef = ( $ColRef =~ /:/ ) ? split ':', $ColRef : ($ColRef, -1) ; # reference column
 my @ColCpy = ( $ColCpy ne ""  ) ? split ':', $ColCpy : (           ) ; # copy columns
    
 printf "# InpFil : $iFile   \n" if $verb > 1 and defined $iFile;
 printf "# OutFil : $oFile   \n" if $verb > 1 and defined $oFile;
 printf "# ColVal : @ColVal  \n" if $verb > 1;
 printf "# ColRms : @ColRms  \n" if $verb > 1;
 printf "# ColRef : @ColRef  \n" if $verb > 1;
 printf "# ColCpy : @ColCpy  \n" if $verb > 1;
 printf "# RunStt : $RunStt  \n" if $verb > 1;
 printf "# %s\n",         '-'x60 if $verb > 1;

 my $colV = $ColVal[0];
 my $colR = $ColRms[0];
 my $colX = $ColRef[0];
      
 # ================================
 # get data (read file/stdinp)
 # ================================
 if( defined $iFile and -f "$iFile" ){ open(IFIL, "<$iFile")
 }else{                                open(IFIL, "<-"     ) }
 
 my $LineIdx = 0;
 while( <IFIL> ){ chomp();
      
   $_ =~ s/^\s+//g;     # delete space at beginning
   if( /^#/ ){ next };  # skip comments
   if( /^$/ ){ next };  # skip empty lines
   
   # split line into the columns and fill the array
   $LineIdx++;
   my( @COL ) = split '\s+', $_;
   unshift( @COL, 0 );           # COL[0] = 1 marked values

   # check columns
   if( scalar @COL > $colX  and  # colX=0 if not defined
       scalar @COL > $colR  and  # colX=0 if not defined
       scalar @COL > $colV  )    # colX=0 if not defined
   {
     # column not defined
     if( $COL[$colV] =~ /\-\-/ )
     {
      $NoVal++; 
#      $TotVal++;
#       $COL[0] = 2; # skip
#     push( );
            
     }elsif( ($ColRms  ne "0" and $COL[$colR] == 0 )  # if checked RMScolumn, check for RMSvalue
	  or ($ColRms[1] > 0 and abs($COL[$colR]) > abs($ColRms[1]) )
	  or ($ColVal[1] > 0 and abs($COL[$colV]) > abs($ColVal[1]) )){
   
       # absolute values exceeded (MaxVal)
#      $ExVal++;
#      $TotVal++;
       $COL[0] = 1; # outlier
		 
       push( @ALL, [ @COL ] );

     }else{

       # value ok (not printed)
#      $OkVal++; 
#      $TotVal++;              	   
       $COL[0] = 0; # ok value
	 
       push( @ALL, [ @COL ] );
     }
   }	       
     
   # printf if any problem with this data was identified
   if( $verb > 0 and $COL[0] > 0 ){
      printf STDERR "%-4i line - %8.8s: %s\n", $LineIdx, "excluded", $_;
   }
 }
 close(IFIL);
 # ================================
 # generate statistics
 # ================================
 my( @OK );
 my $idx;   
 my $check = 1;
 my $StdDevM = 0;
 my $RedSumM = 0;
    
 while( $check ){
     
   $Iter++;

   # re-initialize
   $idx    = 0;
   $OkVal  = 0;
   $ExVal  = 0;
   $TotVal = 0;
   $OkValW = 0;
   $Sum    = 0;
   $SumA   = 0;
   $Sum2   = 0;
   $SumW   = 0;
   $SumV   = 0;
   $RedSum2= 0;
   $RedSumM= 0;
     
   # loop ALL: sorted array/original values
   # - find&mark outliers
   # - sum of values
   # - sum of absolute values
   # - sum of quadrate values
   for my $pt ( @ALL )
   {
     # first exclude previously marked values
     if( $pt->[0] > 0 ){ next };

     # outlier rejection (robust using Median)
     my $chk_Outl = ( $StdDevM*$CnfInt < abs($pt->[$colV]-$Median) );       
     # data already absolute values
     if( $AbsVal > 0 ){ # pouzij jiny CnfInt ne pro normalni rozdeleni (outl jen pro vetsi hodnoty nez mean) !
	                # ci pouzit RMS a neredukovana data
	$chk_Outl = ( $StdDevM*$CnfInt <     $pt->[$colV]-$Median  );
     }
     if( $StdDevM != 0 and $chk_Outl ){
       $pt->[0] = $Iter;
       if( $verb > 0 ){
	 printf STDERR "# rejected $pt->[0] : %6.4f %6.4f %6.4f %6.4f\n", $pt->[$colV], $pt->[$colV] - $Median , $StdDevM, $Median if $verb > 0;
       }
       next;
     }
      
     $idx++;
     $Sum       +=      $pt->[$colV];
     $SumA      += abs( $pt->[$colV] );
     $Sum2      +=      $pt->[$colV] * $pt->[$colV];
   
     # weighted sum (only WRMS)
     if( $colR >= 0 and $pt->[$colR] > 0 ){
       my $K  = 1;  # not relevant now # I will multiply when WRms calculated
       my $W  = $K / ($pt->[$colR] * $pt->[$colR]);
       $SumV += $W * ($pt->[$colV] * $pt->[$colV]);
       $SumW += $W;
       $OkValW++; 
     }
     printf STDERR "%-4i %+12.5f %+12.5f  %+12.5f\n", $idx, $pt->[$colV], $Sum, $Sum2 if $verb > 2;
   }
   # ---- end of loop ALL

   # handle arrays
     
   @OK = ();     
   # select ok values for sorted array (always reduced data from robust Median ! )
   foreach my $pt ( @ALL ){ if( $pt->[0] == 0){ push(@OK, $pt) } }
   my @OK_OriginSort =         sort {    $a->[$colV]  <=>     $b->[$colV]  } @OK;
   my @OK_AbsOrgSort =         sort {abs($a->[$colV]) <=> abs($b->[$colV]) } @OK;
   $TotVal = scalar @ALL;  
    $OkVal = scalar @OK;
    $ExVal = $TotVal - $OkVal;

   if( $OkVal <= 0 ){ printf STDERR "No valid value !\n"; exit(1); }
   
   # central statistics
   # .. median of values
   # .. mean of values
   # .. mean of absolute values
   $MidIdx  = ( $OkVal   )/2;
   $MidIdx  = ( $OkVal+1 )/2  if( $OkVal % 2 );
   $Median  = $OK_OriginSort[$MidIdx-1][$colV];
   $AbsMedi = abs($OK_AbsOrgSort[$MidIdx-1][$colV]);
   $Mean    = ($OkVal > 0) ? $Sum  / $OkVal : 0;
   $AbsMean = ($OkVal > 0) ? $SumA / $OkVal : 0;
     
   # required RefValue !
   if( defined $ValRef ){ $Mean = $ValRef; }
     
   # loop: sorted array/reduced values
   for my $pt ( @OK ){ 
      $RedSum2   += ($pt->[$colV]-$Mean)   * ($pt->[$colV]-$Mean  ); # using mean  (correct)
      $RedSumM   += ($pt->[$colV]-$Median) * ($pt->[$colV]-$Median); # using median (robust) 
   }
 
   # variation statistics
   # .. standard deviation
   # .. root mean square
   # .. weighted root mean square
   $StdDev  = ($OkVal-1 > 0) ? sqrt( $RedSum2 / ($OkVal-1) ) : 0;
   $StdDevM = ($OkVal-1 > 0) ? sqrt( $RedSumM / ($OkVal-1) ) : 0;
   $Rms     = ($OkVal   > 0) ? sqrt(    $Sum2 /  $OkVal    ) : 0;
   if( $ColRms ne "0" ){
     $WRms  = ( $SumW > 0) ? sqrt( $SumV / $SumW ) : 0;       
     # $WRms  = ($OkValW - 1 > 0) ? sqrt( $SumV *$Rms*$Rms / ($OkValW-1) ) : 0;
     # $WRms  = ($OkValW - 1 > 0) ? sqrt( $SumV / ($OkValW-1) ) : 0;
   }
     
   my @OK_ReduceSort =         sort {    $a->[$colV] - $Median  <=>     $b->[$colV] - $Median } @OK;
   my @OK_RedAbsSort = reverse sort {abs($a->[$colV] - $Median) <=> abs($b->[$colV] - $Median)} @OK;

   # iteration ok ?
   # - additionally consider testing  abs($Mean - $Median) <= $StdDevM / 100  # (100 empiric)
   # -------------------------------
   # data already absolute values
   my $chk_Outl = ( $StdDevM != 0 and $StdDevM*$CnfInt < abs($OK_RedAbsSort[0][$colV]-$Median) );
   if( $AbsVal > 0 ){ # pouzij jiny CnfInt ne pro normalni rozdeleni (outl jen pro vetsi hodnoty nez mean) !
	              # ci pouzit RMS a neredukovana data
      $chk_Outl = ( $StdDevM != 0 and $StdDevM*$CnfInt <     $OK_RedAbsSort[0][$colV]-$Median  );
   }
   if( $chk_Outl and $Iter <= $MaxIter ){
     # probably problem with estimating mean from poor data -> re-iterate with Median to be more robust
     $Mean   = $Median;
     if( $verb > 0 ){
        printf STDERR "# Iter $Iter: %7.3f %7.3f - %7.3f : %6.3f   ok: %4i  ex: %3i\n",
            $OK_RedAbsSort[0][$colV], $OK_RedAbsSort[1][$colV], $Median, ( $OK_RedAbsSort[0][$colV] - $Median ), $OkVal, $ExVal;
     }
   }else{
     $check = 0;
   }
 }

 # counting statistics
 # .. percentage of accepted values
 # .. percentage of  missing values
 # .. percentage of excluded values
 $OkPc    = ($TotVal > 0) ? $OkVal / $TotVal * 100  :  0;
 $NoPc    = ($TotVal > 0) ? $NoVal / $TotVal * 100  :  0;
 $ExPc    = ($TotVal > 0) ? $ExVal / $TotVal * 100  :  0;
     
 # running statistics
 # .. running mean    (forward/backward)
 # .. running stddev  (forward/backward)
 # .. running exclusion of the residuals
 if( defined $RunStt and $RunStt >= 0 ){
  
   my $RunIdx = 0;
   my ( $RunSum, $RunMean, $RunSum2, $RunRms ) = ( 0, 0, 0, 0);
   my ( $RunRedSum2,  $RunStdDev  ) = ( 0, 0 );
   
   my $lastIdx = scalar @{$ALL[0]};
#   printf "lastIdx = $lastIdx \n";
   for my $pt ( ($RunStt == 1) ? reverse @ALL : @ALL )
   {
     my $dat = $pt->[$colV];
     
     # used value
     $RunIdx      += 1;
     if( $pt->[0] == 0 ){
       $RunSum      += $dat;
       $RunMean      = $RunSum/($RunIdx);
       $RunSum2     += ($dat*$dat);
       $RunRedSum2  += ($dat-$Mean)*($dat-$Mean);
     
       $RunStdDev    = ($RunIdx-1 > 0) ? sqrt( $RunRedSum2  / ($RunIdx-1) ) : 0;
       $RunRms       = ($RunIdx   > 0) ? sqrt( $RunSum2     /  $RunIdx    ) : 0;
     }
#     $pt->[$lastIdx+0] = $RunMean;       # 3 Running Mean 
#     $pt->[$lastIdx+0] = $dat-$Mean;     # 3 Reduced data  ok for N,E,U, but not for Position
     $pt->[$lastIdx+0] = $dat;           # 3 Data           ok for Position!
     $pt->[$lastIdx+1] = $RunRms;        # 4 Running Rms
     $pt->[$lastIdx+2] = $RunStdDev;     # 5 Running Sdev
     $pt->[$lastIdx+3] = $RunIdx;        # 6 Running Index
   }
 
   # print statistics into file (or stdout if file not defined)
   if( defined $oFile and $oFile ne "" ){ open(OFIL, ">$oFile")     
   }else{                                 open(OFIL, ">-"     ) }

     # print running statistics/all data in original order
        $ConvEpo    = undef;              # Convergence epoch
     my $firstDiff  = undef;              # tDiff after last outlier
     my $firstEpoch = $ALL[0][$colX];     # first epoch
     my $outlCount  = 0;                  # count of all outliers
     my $last       = 0;                  # last value was outlier
     
   for my $pt ( @ALL )
   {
     if( "$CnvLim" =~ "auto" ){
        $CnvLim = $pt->[$lastIdx+2]*$CnfInt;
     }
     my $tDiff = ($pt->[$colX]-$firstEpoch)*24*60;

     # check outlier
     my $outl = "";
     if( ( $pt->[$lastIdx+2] != 0 and
           $CnvLim < abs($pt->[$colV]) ) )               # for Position
#           $CnvLim < abs($pt->[$colV] - $Mean) ) )      # for N,E,U
#        or $pt->[0] > 0 ) # a priori excluded           ! NO !!!!
     {
       # reset first epoch and count
       $firstDiff = undef;
       $last      =     0;
       $outlCount++;

       $outl = sprintf " --- %4.1f %4.1f outlier", $tDiff, $outlCount / $pt->[$lastIdx+3] * 100;
     }else{

       # count a sequence of last values (no outlier between)
       $last++;
	 
       # skip first value
       if( $pt->[$lastIdx+3]  == 1 ){ next }

       # remember first tDiff after outlier
       if( ! defined $firstDiff ){ $firstDiff = $tDiff; }

       # and ($outlCount/$pt->[$lastIdx+3] * 100) < 60

       # identify the convergence time
       if( $last > 50 ){
         $ConvEpo = $firstDiff if ( ! defined $ConvEpo );
	   
         $outl = sprintf " --- %4.1f %4.1f", $tDiff, $ConvEpo;
       }else{
         $outl = sprintf " --- %4.1f", $tDiff;
       }
     }

     # printing " standard columns + added Running values"
     my $print = "";
     for(my $i=1; $i<$lastIdx; $i++ ){ $print .= sprintf " %s", $pt->[$i] }; $print .= " | "; # columns
     
     for(my $i=0; $i<=3;       $i++ ){ $print .= sprintf " %12.4f", $pt->[$lastIdx+$i] }      # columns added

     printf OFIL "%s %s \n", $print, $outl;
   }
   close(OFIL);
 }
  
 $ConvEpo = 0 if ( ! defined $ConvEpo );


 # substitute values
 # ================================
# printf "outFmt = %s \n", $outFmt;
 if( $outFmt =~  /%([+-.\d]*f|)ConvEpo/        ){ my $fmt = ( $1 ne "" ) ? "$1" : "5.1f" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$ConvEpo";
     $outFmt =~ s/%([+-.\d]*f|)ConvEpo/$tmp/g;
 }
    
 if( $outFmt =~  /%([+-.\d]*f|)AbsMedian/      ){ my $fmt = ( $1 ne "" ) ? "$1" : "12.7f" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$AbsMedi";
     $outFmt =~ s/%([+-.\d]*f|)AbsMedian/$tmp/g;
 }
 if( $outFmt =~  /%([+-.\d]*f|)Median/         ){ my $fmt = ( $1 ne "" ) ? "$1" : "12.7f" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$Median";
     $outFmt =~ s/%([+-.\d]*f|)Median/$tmp/g;
 }
 if( $outFmt =~  /%([+-.\d]*f|)AbsMean/        ){ my $fmt = ( $1 ne "" ) ? "$1" : "12.7f" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$AbsMean";
     $outFmt =~ s/%([+-.\d]*f|)AbsMean/$tmp/g;    
 }
 if( $outFmt =~  /%([+-.\d]*f|)Mean/           ){ my $fmt = ( $1 ne "" ) ? "$1" : "12.7f" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$Mean";
     $outFmt =~ s/%([+-.\d]*f|)Mean/$tmp/g;
 }    
 if( $outFmt =~  /%([+-.\d]*f|)StdDev/         ){ my $fmt = ( $1 ne "" ) ? "$1" : "12.7f" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$StdDev";
     $outFmt =~ s/%([+-.\d]*f|)StdDev/$tmp/g  
 }
 if( $outFmt =~  /%([+-.\d]*f|)Rms/            ){ my $fmt = ( $1 ne "" ) ? "$1" : "12.7f" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$Rms";
     $outFmt =~ s/%([+-.\d]*f|)Rms/$tmp/g;
 }
 if( $outFmt =~  /%([+-.\d]*f|)WRms/           ){ my $fmt = ( $1 ne "" ) ? "$1" : "12.7f" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$WRms";
     $outFmt =~ s/%([+-.\d]*f|)WRms/$tmp/g;
 }
 if( $outFmt =~  /%([+-.\d]*i|)Iter/           ){ my $fmt = ( $1 ne "" ) ? "$1" : "12i" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$Iter";
     $outFmt =~ s/%([+-.\d]*i|)Iter/$tmp/g;
 }    
 if( $outFmt =~  /%([+-.\d]*i|)OkVal/          ){ my $fmt = ( $1 ne "" ) ? "$1" : "12i" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$OkVal";
     $outFmt =~ s/%([+-.\d]*i|)OkVal/$tmp/g;      
 }
 if( $outFmt =~  /%([+-.\d]*i|)NoVal/          ){ my $fmt = ( $1 ne "" ) ? "$1" : "12i" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$NoVal";
     $outFmt =~ s/%([+-.\d]*i|)NoVal/$tmp/g;
 }
 if( $outFmt =~  /%([+-.\d]*i|)ExVal/          ){ my $fmt = ( $1 ne "" ) ? "$1" : "12i" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$ExVal";
     $outFmt =~ s/%([+-.\d]*i|)ExVal/$tmp/g;
 }
 if( $outFmt =~  /%([+-.\d]*i|)TotVal/         ){ my $fmt = ( $1 ne "" ) ? "$1" : "12i" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$TotVal";
     $outFmt =~ s/%([+-.\d]*i|)TotVal/$tmp/g;
 }
 if( $outFmt =~  /%([+-.\d]*i|)MidIdx/         ){ my $fmt = ( $1 ne "" ) ? "$1" : "i" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$MidIdx";
     $outFmt =~ s/%([+-.\d]*i|)MidIdx/$tmp/g;
 }
 if( $outFmt =~  /%([+-.\d]*f|)OkPc/           ){ my $fmt = ( $1 ne "" ) ? "$1" : "4.1f" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$OkPc";
     $outFmt =~ s/%([+-.\d]*f|)OkPc/$tmp/g;      
 }
 if( $outFmt =~  /%([+-.\d]*f|)NoPc/           ){ my $fmt = ( $1 ne "" ) ? "$1" : "4.1f" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$NoPc";
     $outFmt =~ s/%([+-.\d]*f|)NoPc/$tmp/g;
 }
 if( $outFmt =~  /%([+-.\d]*f|)ExPc/           ){ my $fmt = ( $1 ne "" ) ? "$1" : "4.1f" ;
                                                  my $tmp = eval "sprintf \"%$fmt\",$ExPc";
     $outFmt =~ s/%([+-.\d]*f|)ExPc/$tmp/g;      
 }
 # substitute Special Characters
 $outFmt =~ s|%n|\n|g        ; #  newline
 $outFmt =~ s|%t|\t|g        ; #  tabulator
 $outFmt =~ s|[%]+|%%|g      ; #  substite doubled to single '%%' -> '%'
    
 return "$outFmt";   
}


# ==================================================================================
# INTERNAL SUBROUTINES

# usage
# -----
sub Usage {    
printf "@_\n";
format STDERR =

 Usage: calc_stat options
      
   < --ifile     input file >   .. input data file
   [ --ofile    output file ]   .. output file
   < --column  col[:MaxVal] >   .. data column + max Value (optionally, use absolute value)
   [ --rmscol  col[:MaxRms] ]   .. RMS column + max RMS (optionally, positive value)
   [ --refcol  col[:col:..] ]   .. reference column for outlier (running statistics)
   [ --refval          real ]   .. reference value for mean
   [ --cpycol  col[:col:..] ]   .. list of copy column(s) (valid for running statistics)
   [ --cnvlim    'auto'|int ]   .. convergence limit (auto or integer)
   [ --cnfint          real ]   .. confident interval for outlier rejection
   [ --absval               ]   .. data are already in absolute values
   [ --outfmt        outfmt ]   .. optional format for the output (see bellow)
   [ --run              0|1 ]   .. add running statistics (0=forward, 1=reverse)
   [ --verb                 ]   .. verbosity level
   [ --help                 ]   .. this help

 KeyWords:  %Median, %Mean, %AbsMean, %StdDev, %Rms, %WRms  # statistics
            %TotVal, %OkVal, %ExVal, %NoVal                 # counts
            %OkPc, %ExPc, %NoPc                             # percentages
      
 Formating: fmt  ... %+3.2fKeyWord  = "e.g. %5.2fMedian, %+7iOkVal"
            spec ... %t %n %%       = "\tab" "\newline" "%"
            pads ... %: %_          = "any_pad","blank_pad", (default="0_pad")x

.

  write STDERR;
  exit 1;
}

1;
__END__
