#!/usr/bin/perl -w

#=============================================================================#
#                                                                             #
# Name    : LDC_list.pm                                                       #
#                                                                             #
# Purpose : Routines for LDC_lists                                            #
#                                                                             #
# Author  : J. Dousa                                                          #
#                                                                             #
# Created : 11 Jan 2005                     Updated:   11 Jan 2005            #
#                                                                             #
# Changes :                                                                   #
#=============================================================================#

package LDC_list;

use lib "$ENV{HOME}/bin";
use lib "$ENV{HOME}/bin/perl_modules";
use Math::Trig;

require Exporter;
@ISA       = qw( Exporter );
@EXPORT    = qw( &get_LatLonHgt &get_SiteList
	       );
$|         = 1;

use constant  PI => 4 * atan2(1,1);
my $Major_axis = 6378136.55;  # my $Major_axis = 6378137.000
my $Flattening =  298.25769;  # my $Flattening = 298.2572236;
my $flat = $Flattening;
my $Minor_axis = $Major_axis - $Major_axis / $flat;
my $e2 = ($Major_axis**2 - $Minor_axis**2) / ( $Major_axis**2);

# -------------------------------------------------------------------
# EXPORTED FUNCTIONS
# -------------------------------------------------------------------

# -------------------------------------------------------------------
# Get site list
# =============================
# index site list
# ---------------------------
sub get_SiteList {
 my $list = shift || return -1;
 my @Item = @_;
 my @Idx;
 my %LST;

 my $LabSit = 'CODE';
 my $LabDmx = 'Domex____';
 my $LabDsc = 'Site_Name___________';
 my $LabCou = 'Country,';
 my $LabRec = 'Receiver____________';
 my $LabAnt = 'Antenna________';
 my $LabRad = 'dome';
 my $LabLat = 'Latitude';
 my $LabLon = 'Longitud';
 my $LabHel = '__El_Hgt';
    
 # get index positions from the LDC list
 my( $irc, $PosSit, $PosDmx, $PosDsc, $PosCou, $PosRec, $PosAnt, $PosRad, $PosLat, $PosLon, $PosHel )
   = &get_position( $list, $LabSit, $LabDmx, $LabDsc, $LabCou, $LabRec, $LabAnt, $LabRad, $LabLat, $LabLon, $LabHel );

 open( LIST, "$list" ) or die 'not valid LDC list';
 while( $line = <LIST> ){

   if( $line !~ /^[\d]/ ){ next }  # skip line not starting with number

                $site = substr( $line, $PosSit,  length($LabSit) );
   $LST{$site}{"dmx"} = substr( $line, $PosDmx,  length($LabDmx) );
   $LST{$site}{"dsc"} = substr( $line, $PosDsc,  length($LabDsc) );
   $LST{$site}{"cou"} = substr( $line, $PosCou,  length($LabCou) );
   $LST{$site}{"rec"} = substr( $line, $PosRec,  length($LabRec) );
   $LST{$site}{"ant"} = substr( $line, $PosAnt,  length($LabAnt) );
   $LST{$site}{"rad"} = substr( $line, $PosRad,  length($LabRad) );
   $LST{$site}{"lat"} = substr( $line, $PosLat,  length($LabLat) );
   $LST{$site}{"lon"} = substr( $line, $PosLon,  length($LabLon) );
   $LST{$site}{"hel"} = substr( $line, $PosHel,  length($LabHel) );
     
   $lat = $LST{$site}{"lat"} * PI/180;
   $lon = $LST{$site}{"lon"} * PI/180;
   $hel = $LST{$site}{"hel"} * PI/180;
   $N   = $Major_axis / sqrt( 1 - $e2*sin($lat));
    
   $LST{$site}{"X"}   =  ($N         + $hel) * cos($lat) * cos($lon);
   $LST{$site}{"Y"}   =  ($N         + $hel) * cos($lat) * sin($lon);
   $LST{$site}{"Z"}   =  ($N*(1-$e2) + $hel) * sin($lat);
 } 
 return \%LST;
}

# -------------------------------------------------------------------
# Get coordinates from site-list
# =============================
sub get_LatLonHgt {
 my $site  = shift;
 my @lists = @_;
 my( $Lat, $Lon, $Hgt );
    
LIST:
 foreach my $list ( @lists ){
     
   my( $irc, $PosLat, $PosLon, $PosHgt, $PosX, $PosY, $PosZ )
     = &get_position( $list, 'LATITUDE', 'LONGITUD', '__EL_HGT', '_X_____________', '_Y_____________', '_Z_____________' );
     
   if( $irc ){ next }

   my $xyz = 0;
   my $blh = 0;

   open( LIST, "$list" );
   while( <LIST> ){

     if( uc($_)=~/LATITUD/ ){ $blh = 1; $xyz = 0; next }
     if( uc($_)=~/_X_____/ ){ $blh = 0; $xyz = 1; next }

     if( / $site / ){
       if( $xyz eq 1 ){
         $X = substr( $_, $PosX, 16 );
         $Y = substr( $_, $PosY, 16 );
         $Z = substr( $_, $PosZ, 16 );

	 ($Lat, $Lon, $Hgt) = xyz2blh( $X, $Y, $Z );
	  $Lat = sprintf "%+6.2f", $Lat*180/PI;
	  $Lon = sprintf "%+6.2f", $Lon*180/PI;
	  $Hgt = sprintf "%+6.2f", $Hgt*180/PI;
       }
       if( $blh eq 1 ){
         $Lat = substr( $_, $PosLat, 8 );
         $Lon = substr( $_, $PosLon, 8 );
         $Hgt = substr( $_, $PosHgt, 8 );
       }
     }
   } 
   close( LIST );
     
   if( defined $Lat  and
       defined $Lon  and
       defined $Hgt      ){  return( $Lat, $Lon, $Hgt ) }
 }
    
 return( 99999, 99999, 99999 );
}

# -------------------------------------------------------------------
# INTERNAL FUNCTIONS
# -------------------------------------------------------------------

# index position for a string
# ---------------------------
sub get_position {
 my $List = shift;
 my @Item = @_;
 my @Idx;
    
 open( LIST, "$List" ) or return 1;
    
LINE:
 while( <LIST> ){
    
  if( $_ =~ /^[\d ]/ ){ next LINE }
     
   foreach my $item ( @Item ){
       
#     my $idx = index( uc($_), uc($item) );
     my $idx = index( $_, $item );
     push( @Idx, $idx );       
   }
   last LINE;
 }
 close( LIST );
 return 0, @Idx;
}

# conversion XYZ -> BLH
# ---------------------
sub xyz2blh(){
     my( $x, $y, $z ) = @_;
    
     my $s = sqrt( $x**2 + $y**2 );
     my $r = sqrt( $x**2 + $y**2  + $z**2 );
    
     my $l   = atan2( $y, $x );                           #  printf "l = %10.5f  ", $l / PI*180;
     if( $l lt 0 ){ $l = 2*PI + $l };                     #  printf "/ %10.5f    ", $l / PI*180;
    
     my $zps = $z / $s;                                   #  printf "zps = %10.5f ", $zps;
     my $h   = $r - $Major_axis;                          #  printf "h = %10.5f   ", $h;
     my $b   = atan2( $zps,(1 - $e2 * $Major_axis/$r));   #  printf "b = %10.5f   ", $b / PI*180;
     my $hdif = 0;
     my $bdif = 0;
     my $iter = 0;                                        #  printf "\n";
     do{
	    my $n  = $Major_axis / sqrt(1 - $e2 * sin($b)**2);
	    my $hi = $h;
	    my $bi = $b;
	       $h  = $s / cos( $b ) - $n;
	       $b  = atan( $zps / (1 - $e2 * $n / ($n+$h) ));
	 
	       $hdif = abs( $hi - $h );
	       $bdif = abs( $bi - $b );                         #  printf "$iter $bdif $hdif \n";
	       $iter++;
	  }while( $bdif >= 0.00000000001 or $hdif >= 0.00005 );
    
     $b = sprintf "%+12.10f", $b;
     $l = sprintf "%+12.10f", $l;
     $h = sprintf "%+7.3f",   $h;
     return ( $b, $l, $h );
}



__END__




